/* $Id: network.c 579 2014-05-17 21:23:44Z bhockney $ */

/* (C) 2004-2014 by Bob Hockney <zeus@ix.netcom.com>    *
 * Based on fwlogwatch written by                       *
 * Boris Wesslowski <Wesslowski@CERT.Uni-Stuttgart.DE>  *
 *                                                      *
 * network utility routines for wfwl_syslog             *
 *                                                      *
 * This code is distributed under the terms of GNU GPL  */

#include <unistd.h>
#include <string.h>
#include <syslog.h>
#include <errno.h>
#include <sys/wait.h>
#include <netdb.h>

#include <main.h>
#include <network.h>

struct options opt;

/* xstrncpy() - similar to strncpy(3) but always terminates string *
 * with '\0' (if n > 0 and dest != NULL),  doesn't do padding.     */
char *xstrncpy(char *dest, const char *src, size_t n)
{
  char *r = dest;

  if((n <= 0) || (dest == NULL)) {
    return dest;
  }
  if(src != NULL) {
    while ((--n != 0) && (*src != '\0')) {
      *dest++ = *src++;
    }
  }
  *dest = '\0';
  return r;
}

/* Returns OK if ip address string ip is valid for address *
 * family.  If address is valid, also sets addr or addr6   *
 * as appropriate and copies ip to ip_addr_str, shortened  *
 * for AF_INET6 as returned by inet_ntop, .                  */
unsigned char convert_ip(int family, char *ip, struct in_addr *addr, void *addr6, char *ip_addr_str) 
{
#if HAVE_INET_ATON || HAVE_INET_PTON
  int retval;
#endif

  if (family == AF_INET) {
#ifdef HAVE_INET_ATON
  retval = inet_aton(ip, addr);
  if (retval == 0) {
#else
#ifndef INADDR_NONE
#define INADDR_NONE -1
#endif
  addr->s_addr = inet_addr(ip);
  if (addr->s_addr == INADDR_NONE) {
#endif
    if (opt.verbose >= VERBOSE_NOTICE)
      fprintf(stderr, "IP address error: %s\n", ip);
    return IN_ADDR_ERROR;
  }
  xstrncpy(ip_addr_str, ip, INET6_ADDRSTRLEN);
  return IN_ADDR_OK;
#ifdef HAVE_INET_PTON
  } else if (family == AF_INET6) {
    retval = inet_pton(AF_INET6, ip, addr6);
    if (retval == 0) {
      if (opt.verbose >= VERBOSE_NOTICE)
        fprintf(stderr, "IP address error: %s\n", ip);
      return IN_ADDR_ERROR;
    }
    if (ip_addr_str != NULL)
      inet_ntop(AF_INET6, addr6, ip_addr_str, INET6_ADDRSTRLEN);
    return IN_ADDR_OK;
#endif
  }
  return IN_ADDR_ERROR;
}

/* Compares memory locations l and r for n bits. */
/* Used to compare IPv6 addresses in memory.     */
/* Returns 0 if a match, non-zero otherwise.     */
/* Assumes network byte order.                   */

int bitncmp(void *l, void *r, int n) {
  unsigned int lb, rb;
  int x, b;

  b = n / 8;
  x = memcmp(l, r, b);
  if (x || (n % 8) == 0)
    return x;

  lb = ((const unsigned char *) l)[b];
  rb = ((const unsigned char *) r)[b];
  for (b = n % 8; b > 0; b--) {
    if (((unsigned char)lb & 0x80) != ((unsigned char)rb & 0x80))
      return 1;
    
    lb <<= 1;
    rb <<= 1;
  }

  return 0;
}
